#ifndef CRYPTOPRO_BACKEND_H
#define CRYPTOPRO_BACKEND_H

#include "CryptoSignBackend.h"
#include "pdfcsp/csp_for_poppl.hpp"

class CryptoProSignatureVerification : public CryptoSign::VerificationInterface
{
public:
    explicit CryptoProSignatureVerification(std::vector<unsigned char> const& pkcs7data);
    SignatureValidationStatus validateSignature() final;
    SignatureValidationStatus validateSignature(std::string& pdfFile,
            std::vector<std::pair<uint64_t, uint64_t>>&& byteRanges) final;
    void addData(unsigned char* dataBlock, int dataLen) final;
    std::chrono::system_clock::time_point getSigningTime() const final;
    std::string getSignerName() const final;
    std::string getSignerSubjectDN() const final;
    HashAlgorithm getHashAlgorithm() const final;
    CertificateValidationStatus validateCertificate(std::chrono::system_clock::time_point validation_time,
            bool ocspRevocationCheck, bool useAIACertFetch) final;
    std::unique_ptr<X509CertificateInfo> getCertificateInfo() const final;

private:
    pdfcsp::poppler::BytesVector m_pkcs7data;
    std::unique_ptr<pdfcsp::poppler::ESInfo> m_esinfo;
};

class CryptoProSignBackend final : public CryptoSign::Backend
{
public:
    std::unique_ptr<CryptoSign::VerificationInterface> createVerificationHandler(std::vector<unsigned char>&& pkcs7) final;
    std::unique_ptr<CryptoSign::SigningInterface> createSigningHandler(const std::string &certID, HashAlgorithm digestAlgTag) final;
    std::vector<std::unique_ptr<X509CertificateInfo>> getAvailableSigningCertificates() final;
    ~CryptoProSignBackend() final;
};

#endif // CRYPTOPRO_BACKEND_H
